@@ -1,6 +1,6 @@
 /*                                               -*- c-file-style: "ruby"; -*-
  * mod_img - an Image Manipulatively Gadget for Apache
- * $Id: mod_img.c,v 1.5 2003/10/03 07:38:15 yuya_n Exp $
+ * $Id: mod_img.c,v 1.8 2003/10/21 11:23:43 yuya_n Exp $
  * 
  * Copyright (C) 2003  Yuya.Nishida. <yuya at@at j96 dot.dot org>
  * All rights reserved.
@@ -43,7 +43,7 @@
 #include "ap_config.h"
 #include "http_request.h"
 
-#include <acconfig.h>
+#include <config.h>
 
 
 static const char *
@@ -572,6 +572,74 @@ is_no_change(request_rec *r)
     return 1;
 }
 
+static table *
+parse_accept(pool *p, const char *accept)
+{
+    table *accept_table = ap_make_table(p, 10);
+    const char *value;
+    char *tmp;
+
+    if (accept == NULL) {
+        return accept_table;
+    }
+
+    value = tmp = ap_pstrdup(p, accept);
+    for (; tmp[0] != '\0'; tmp++) {
+        switch (tmp[0]) {
+          case ';':
+            tmp[0] = '\0';
+            ap_table_set(accept_table, value, value);
+            tmp = strchr(tmp, ',');
+            if (tmp == NULL) {
+                return accept_table;
+            }
+            value = tmp + 1;
+            break;
+          case ' ':
+          case ',':
+            tmp[0] = '\0';
+            ap_table_set(accept_table, value, value);
+            value = tmp + 1;
+            break;
+        }
+    }
+    ap_table_set(accept_table, value, value);
+
+    return accept_table;
+}
+
+static const OutputType *
+get_client_accept_output_type(request_rec *r, const OutputType *output_type)
+{
+    int i;
+    const char *accept = ap_table_get(r->headers_in, "Accept");
+    table *accept_table;
+
+    if (accept == NULL) {
+        return output_type;
+    }
+
+    accept_table = parse_accept(r->pool, accept);
+
+    if (ap_table_get(accept_table, output_type->mime_type) != NULL) {
+        return output_type;
+    }
+
+    for (i = 0; OUTPUT_TYPES[i].mime_type != NULL; i++) {
+        if (!OUTPUT_TYPES[i].save_support) {
+            continue;
+        }
+        if (ap_table_get(accept_table, OUTPUT_TYPES[i].mime_type) != NULL) {
+            ap_log_rerror(APLOG_MARK, APLOG_DEBUG | APLOG_NOERRNO, r,
+                          "get_client_accept_output_type: check(mime_type=%s)",
+                          OUTPUT_TYPES[i].mime_type);
+            return &OUTPUT_TYPES[i];
+        }
+    }
+
+    return output_type;
+}
+
 static ImgConfig *
 get_merged_config(request_rec *r)
 {
@@ -603,6 +671,8 @@ get_merged_config(request_rec *r)
         }
     }
 
+    cfg->output_type = get_client_accept_output_type(r, cfg->output_type);
+
     return (ImgConfig *)cfg;
 }
 
